import jax.numpy as jnp
from functools import partial 


class Beam: pass

def beam(rel=None):
    if rel is None:
        ah = jnp.eye(3)
    else:
        ah = _ah(rel)



    def f(u,xyz,E,A,I):
        """return element local stiffness Matrix"""
        DX = xyz[1,0] - xyz[0,0]
        DY = xyz[1,1] - xyz[0,1]
        L = jnp.linalg.norm([DX,DY])

        EI = E*I
        k = jnp.array([[E*A/L,    0   ,   0   ],
                       [  0  , 4*EI/L , 2*EI/L],
                       [  0  , 2*EI/L , 4*EI/L]])
        f = ah.T @ k @ ah @ u
        return f
    obj = Beam()
    obj.f = f
    obj.Df = Df
    return  obj

def _ah(rel):
    MR = [1 if x else 0 for x in rel]
    ah = jnp.array([[1-MR[0],          0          ,             0        ],
                   [   0   ,       1-MR[1]       ,  -0.5*(1-MR[2])*MR[1]],
                   [   0   , -0.5*(1-MR[1])*MR[2],          1-MR[2]     ]])
    return ah


def beam_2d_stiffness(rel=None,**kwds):
    if rel is None:
        ah = jnp.eye(3)
    else:
        ah = _ah(rel)

    def Df(u,xyz,E,A,I):
        """return element local stiffness Matrix"""
        DX = xyz[1,0] - xyz[0,0]
        DY = xyz[1,1] - xyz[0,1]
        L = jnp.linalg.norm([DX,DY])

        EI = E*I
        k = jnp.array([[E*A/L,    0   ,   0   ],
                        [  0  , 4*EI/L , 2*EI/L],
                        [  0  , 2*EI/L , 4*EI/L]])
        k = ah.T @ k @ ah
        return k

    return partial(Df,**kwds)    


def linear_transform(xyz):
    L = jnp.linalg.norm(xyz)
    cs, sn = (xyz[1] - xyz[0])/L
    ag = jnp.array([[ -cs ,  -sn , 0,  cs ,   sn , 0],
                   [-sn/L,  cs/L,  1, sn/L, -cs/L, 0],
                   [-sn/L,  cs/L,  0, sn/L, -cs/L, 1]])
    return ag




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import io, os, re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import relpath
from os.path import splitext

from setuptools import Extension
from setuptools import find_packages
from setuptools import setup
from setuptools.command.build_ext import build_ext


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()


# Enable code coverage for C code; `FLAGS=-coverage` cannot be used in tox.ini, since that may interfere with compiling
# dependencies. Rather, set `SETUPPY_CFLAGS=-coverage` in tox.ini and copy it to CFLAGS here (after
# deps have been safely installed).
if 'TOXENV' in os.environ and 'SETUPPY_CFLAGS' in os.environ:
    os.environ['CFLAGS'] = os.environ['SETUPPY_CFLAGS']


extras_require = {
    'shellcomplete': ['click_completion'],
    'tensorflow':    ['tensorflow~=2.0', 'tensorflow-probability~=0.10'],
    'jax': ['jax~=0.1,>0.1.72', 'jaxlib~=0.1,>0.1.51']
}
extras_require['backends'] = sorted(
    set(extras_require['tensorflow'] + extras_require['jax'])
)





setup(
    name='elle',
    version='0.0.1',
    description='Library of anonymous finite elements with analytic derivatives.',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.md')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.md'))
    ),
    long_description_content_type= 'text/markdown',
    author='Claudio M. Perez',
    author_email='claudio_perez@berkeley.edu',
    url='https://github.com/claudioperez/elle',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
    project_urls={
        'Changelog': 'https://github.com/claudioperez/elle/blob/master/CHANGELOG.md',
        'Issue Tracker': 'https://github.com/claudioperez/elle/issues',
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    python_requires='>=3.6',
    install_requires=[
        # eg: 'aspectlib==1.1.1', 'six>=1.7',
    ],
    extras_require=extras_require,
)
